#COMPILE DLL                        ' create a dll
OPTION EXPLICIT                     ' require all variables to be declared
#INCLUDE "WIN32API.INC"             ' required equates and windows prototypes
#INCLUDE "MSXStruc.BAS"             ' MSX Data Structures

DECLARE FUNCTION CalculateHilbertPeriod (a_psPrice AS MSXDataInfoRec PTR, _ 'BYVAL a_iFirstValid AS INTEGER, BYVAL a_iLastValid AS INTEGER, _
                                         a_psPeriod AS SINGLE PTR, _
                                         BYVAL a_iArraySize AS INTEGER) AS LONG

'------------------------------------------------------------------------
'
' MSXInfo
'
' Required MSX function.
'
'------------------------------------------------------------------------

FUNCTION MSXInfo SDECL ALIAS "MSXInfo" (a_psDLLDef AS MSXDLLDef PTR) EXPORT AS LONG

   'Copyright Information
   @a_psDLLDef.szCopyright = "Copyright (c) Equis International 2000"
   'Set number of functions to export
   @a_psDLLDef.iNFuncs = 3 'Three Calculation functions
   @a_psDLLDef.iVersion = %MSX_VERSION
   MSXInfo = %MSX_SUCCESS

END FUNCTION


'------------------------------------------------------------------------
'
' MSXNthFunction
'
' Required MSX function.
'
' MSXNthFunction is called once FOR each FUNCTION specified IN the
'   MSXInfo function.  ON each CALL, a_iNthFunc will indicate the
'   zero-based FUNCTION TO define.
' The value copied into a_psFuncDef.szFunctionName must EXACTLY match
'   the exported NAME of the FUNCTION.
'
'------------------------------------------------------------------------

FUNCTION MSXNthFunction SDECL ALIAS "MSXNthFunction" ( BYVAL a_iNthFunc AS LONG, _
                                                       a_psFuncDef AS MSXFuncDef PTR) _
                                                       EXPORT AS LONG
    MSXNthFunction = %MSX_SUCCESS

    SELECT CASE a_iNthFunc
    ' One CASE entry (starting AT 0) exists FOR each exported function.
    CASE 0 ' First Function
        @a_psFuncDef.szFunctionName = "EnhancedHilbertPeriod"
        @a_psFuncDef.szFunctionDescription = "Enhanced Hilbert Cycle Period Function"
        @a_psFuncDef.iNArguments = 1
    CASE 1 ' Second
        @a_psFuncDef.szFunctionName = "ChannelHigh"
        @a_psFuncDef.szFunctionDescription = "Enhanced Hilbert Channel Breakout High Function"
        @a_psFuncDef.iNArguments = 3
    CASE 2 ' Third
        @a_psFuncDef.szFunctionName = "ChannelLow"
        @a_psFuncDef.szFunctionDescription = "Enhanced Hilbert Channel Breakout Low Function"
        @a_psFuncDef.iNArguments = 3
    CASE ELSE
        MSXNthFunction = %MSX_ERROR
    END SELECT

END FUNCTION

'------------------------------------------------------------------------
'
' MSXNthArg
'
' Required FUNCTION only IF one OR more exported functions require
'   arguments. Functions AND arguments are numbered FROM 0.
'
'
'------------------------------------------------------------------------

FUNCTION MSXNthArg SDECL ALIAS "MSXNthArg" ( BYVAL a_iNthFunc AS LONG, BYVAL a_iNthArg AS LONG, _
                                             a_psFuncArgDef AS MSXFuncArgDef PTR) EXPORT AS LONG

    MSXNthArg = %MSX_SUCCESS

    @a_psFuncArgDef.iNCustomStrings = 0

    SELECT CASE a_iNthFunc
    CASE 0 'EnhancedHilbertPeriod Function
        SELECT CASE a_iNthArg
        CASE 0 'EnhancedHilbertPeriod:Arg0
             @a_psFuncArgDef.iArgType = %MSXDataArray
             @a_psFuncArgDef.szArgName = "PriceDataArray"
        CASE ELSE
             MSXNthArg = %MSX_ERROR
        END SELECT
    CASE 1 'ChannelHigh
        SELECT CASE a_iNthArg
        CASE 0 'ChannelHigh:Arg0
            @a_psFuncArgDef.iArgType = %MSXDataArray
            @a_psFuncArgDef.szArgName = "UserPriceArray"
        CASE 1 'ChannelHigh:Arg1
            @a_psFuncArgDef.iArgType = %MSXNumeric
            @a_psFuncArgDef.szArgName = "PeriodMultiplier"
        CASE 2 'ChannelHigh:Arg2
            @a_psFuncArgDef.iArgType = %MSXNumeric
            @a_psFuncArgDef.szArgName = "AlternatePeriod"
        CASE ELSE
             MSXNthArg = %MSX_ERROR
        END SELECT
    CASE 2 'ChannelLow
        SELECT CASE a_iNthArg
        CASE 0 'ChannelLow:Arg0
            @a_psFuncArgDef.iArgType = %MSXDataArray
            @a_psFuncArgDef.szArgName = "UserPriceArray"
        CASE 1 'ChannelLow:Arg1
            @a_psFuncArgDef.iArgType = %MSXNumeric
            @a_psFuncArgDef.szArgName = "PeriodMultiplier"
        CASE 2 'ChannelLow:Arg2
            @a_psFuncArgDef.iArgType = %MSXNumeric
            @a_psFuncArgDef.szArgName = "AlternatePeriod"
        CASE ELSE
             MSXNthArg = %MSX_ERROR
        END SELECT
    END SELECT

END FUNCTION

' ************************************************************************
' This local utility function is used to help ensure that no overflows
'   or underflows will occur during calcuations.  The MSXTest program
'   Stress Test function will call your DLL with a wide range of values,
'   including positive and negative values of FLT_MAX and FLT_MIN.
' Perform all intermediate calculations using doubles and then force the
'   results into the range of a single.
' ************************************************************************
FUNCTION ForceFloatRange (BYVAL a_lfDbl AS DOUBLE) AS DOUBLE

    LOCAL s_MaxSingle AS DOUBLE
    LOCAL s_MinSingle AS DOUBLE
    s_MaxSingle = 3.371E+38
    s_MinSingle = 8.431E-37

    ' If it's a positive number
    IF (a_lfDbl > 0#) THEN
        ' force the positive number to <= to s_MaxSingle
        a_lfDbl = MIN (a_lfDbl, s_MaxSingle)
        ' force the positive number to >= to x_MinSingle
        a_lfDbl = MAX (a_lfDbl, s_MinSingle)
    ' If it's negative number
    ELSE
        IF (a_lfDbl < 0#) THEN
            ' force negative number to >= -s_MaxSingle
            a_lfDbl = MAX (a_lfDbl, -s_MaxSingle)
            ' force negative number to <= -s_MaxSingle
            a_lfDbl = MIN(a_lfDbl, -s_MinSingle)
        END IF
    END IF

    ForceFloatRange = a_lfDbl

END FUNCTION

'///////////////////////////////////////////////////////////////////////
'
' Function: EnhancedHilbertPeriod
'
' Description:
' The Enhanced Hilbert Cycle Period Function used by the Enhanced Hilbert
' Channel Breakout system
'
' Arguments:
' PriceDataArray - Array of price Data (high, low, Close, etc.)
'                  TO calculate cycle period From
'
'///////////////////////////////////////////////////////////////////////


FUNCTION EnhancedHilbertPeriod SDECL ALIAS "EnhancedHilbertPeriod" (a_psDataRec AS MSXDataRec PTR, _
                                    a_psDataInfoArgs AS MSXDataInfoRecArgsArray PTR, _
                                    a_psNumericArgs AS MSXNumericArgsArray PTR, _
                                    a_psStringArgs AS MSXStringArgsArray PTR, _
                                    a_psCustomArgs AS MSXCustomArgsArray PTR, _
                                    a_psResultRec AS MSXResultRec PTR) EXPORT AS LONG

    LOCAL l_bRtrn AS LONG
    l_bRtrn = %MSX_SUCCESS

    ' Variables to hold a_psDataInfoArgs(0)
    LOCAL l_psPrice AS MSXDataInfoRec PTR
    LOCAL l_iFirstValid AS INTEGER
    LOCAL l_iLastValid AS INTEGER
    LOCAL l_bResult AS LONG
    LOCAL l_iArray_Size AS INTEGER
    LOCAL i AS INTEGER

    l_bResult = %MSX_SUCCESS

    ' If the right number of args are passed in and the array is not empty
    IF (@a_psDataInfoArgs.iNRecs = 1 AND _
        @a_psNumericArgs.iNRecs = 0 AND _
        @a_psStringArgs.iNRecs = 0 AND _
        @a_psCustomArgs.iNRecs = 0 AND _
        @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid >= 0 AND _
        @a_psDataInfoArgs.@psDataInfoRecs(0).iLastValid > @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid +6) THEN

        ' Then assign the array to local values
        l_psPrice = @a_psDataInfoArgs.psDataInfoRecs(0)
        l_iFirstValid = @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid
        l_iLastValid = @a_psDataInfoArgs.@psDataInfoRecs(0).iLastValid
        l_iArray_Size = @l_psPrice.iLastValid + 1

        'Create a spot in memory for the Period Data
        DIM l_asPeriod(l_iArray_Size) AS SINGLE
        DIM l_psPeriod AS SINGLE PTR

        l_psPeriod = VARPTR(l_asPeriod(0))

        ' Call the Period Calculation Function
        l_bResult = CalculateHilbertPeriod(@l_psPrice[0], _
                                           @l_psPeriod[0], _
                                           l_iArray_Size)

        ' Store results in the a_psResultRec structure
        FOR i = l_iFirstValid +6 TO l_iLastValid
            @a_psResultRec.@psResultArray.@pfValue[i] = l_asPeriod(i)
        NEXT i

        @a_psResultRec.@psResultArray.iFirstValid = l_iFirstValid + 6
        @a_psResultRec.@psResultArray.iLastValid = l_iLastValid

    ELSE
        ' We've recieved invalid arguments
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Invalid Argument(s)"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    IF (l_bResult <> %MSX_SUCCESS) THEN
        ' CalculateHilbertPeriod Function Failed
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Hilbert Period Calculation Failed"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    EnhancedHilbertPeriod = l_bRtrn

 END FUNCTION


'///////////////////////////////////////////////////////////////////////
'
' Function: CalculateHilbertPeriod
'
' Description:
' Performs the actual calculation of the Hilbert Cycle Period
' called from the Period, High or Low functions
'
' Arguments:
' a_psPrice AS SINGLE PTR  - Array of Price Data to use in calculation
' a_iFirstValid AS INTEGER - First valid element in a_pfPrice
' a_iLastValid AS INTEGER  - Last valid element in a_pfPrice
' a_psPeriod AS SINGLE PTR - Array of Period Data to be populated
' a_iArraySize AS INTEGER  - Actual size of a_pfPeriod
'
'///////////////////////////////////////////////////////////////////////

FUNCTION CalculateHilbertPeriod (a_psPrice AS MSXDataInfoRec PTR, _
                                 a_psPeriod AS SINGLE PTR, _
                                 BYVAL a_iArraySize AS INTEGER) AS LONG
    LOCAL l_bRtrn AS LONG
    l_bRtrn = %MSX_SUCCESS

    'declare local calculation variables
    LOCAL l_iCurrentBar AS INTEGER
    LOCAL l_iPreviousBar AS INTEGER  'Stores l_iCurrentBar - 1
    DIM l_pdSmoother(a_iArraySize) AS DOUBLE
    DIM l_pdDetrender(a_iArraySize) AS DOUBLE
    DIM l_pd_I1(a_iArraySize) AS DOUBLE
    DIM l_pd_Q1(a_iArraySize) AS DOUBLE
    DIM l_pd_jI(a_iArraySize) AS DOUBLE
    DIM l_pd_jQ(a_iArraySize) AS DOUBLE
    DIM l_pd_I2(a_iArraySize) AS DOUBLE
    DIM l_pd_Q2(a_iArraySize) AS DOUBLE
    DIM l_pd_X1(a_iArraySize) AS DOUBLE
    DIM l_pd_X2(a_iArraySize) AS DOUBLE
    DIM l_pd_Y1(a_iArraySize) AS DOUBLE
    DIM l_pd_Y2(a_iArraySize) AS DOUBLE
    DIM l_pd_Re(a_iArraySize) AS DOUBLE
    DIM l_pd_Im(a_iArraySize) AS DOUBLE
    LOCAL PI AS DOUBLE

    PI = 3.14159265359

    ' Main Calculation loop (begins after the fifth valid bar)
    FOR l_iCurrentBar = @a_psPrice.iFirstValid + 5 TO @a_psPrice.iLastValid - 1

        ' Set Previous Bar counter
        l_iPreviousBar = l_iCurrentBar - 1

        ' Calculate Smoother and Detrender values
        l_pdSmoother(l_iCurrentBar) =(4 * @a_psPrice.@pfValue[l_iCurrentBar] + 3 * @a_psPrice.@pfValue[l_iPreviousBar] + _
                                      2 * @a_psPrice.@pfValue[l_iCurrentBar - 2] + @a_psPrice.@pfValue[l_iCurrentBar - 3]) / 10
        l_pdDetrender(l_iCurrentBar) = (.25 * l_pdSmoother(l_iCurrentBar) + .75 * l_pdSmoother(l_iCurrentBar - 2) - _
                                        .75 * l_pdSmoother(l_iCurrentBar - 4) - .25 * l_pdSmoother(l_iCurrentBar - 6))* _
                                        (.046 * @a_psPeriod[l_iPreviousBar] + .332)
        ' Calculate InPhase and Quadrature components
        l_pd_Q1(l_iCurrentBar) = (.25 * l_pdDetrender(l_iCurrentBar) + .75 * l_pdDetrender(l_iCurrentBar - 2) - _
                                  .75 * l_pdDetrender(l_iCurrentBar - 4) - .25 * l_pdDetrender(l_iCurrentBar - 6)) * _
                                  (.046 * @a_psPeriod[l_iPreviousBar] + .332)
        l_pd_I1(l_iCurrentBar) = l_pdDetrender(l_iCurrentBar - 3)

        ' Advance the phase of l_pd_I1 and l_pd_Q1 by 90 degrees
        l_pd_jI(l_iCurrentBar) = .25 * l_pd_I1(l_iCurrentBar) + .75 * l_pd_I1(l_iCurrentBar - 2) - _
                                 .75 * l_pd_I1(l_iCurrentBar - 4) - .25 * l_pd_I1(l_iCurrentBar - 6)
        l_pd_jQ(l_iCurrentBar) = .25 * l_pd_Q1(l_iCurrentBar) + .75 * l_pd_Q1(l_iCurrentBar - 2) - _
                                 .75 * l_pd_Q1(l_iCurrentBar - 4) -.25 * l_pd_Q1(l_iCurrentBar - 6)

        ' Phasor addition to equalize amplitude due to quadrature
        ' calculations (and 3 bar averaging)
        l_pd_I2(l_iCurrentBar) = l_pd_I1(l_iCurrentBar) - l_pd_jQ(l_iCurrentBar)
        l_pd_Q2(l_iCurrentBar) = l_pd_Q1(l_iCurrentBar) + l_pd_jI(l_iCurrentBar)

        ' Smooth the I and Q components before applying the discriminator
        l_pd_I2(l_iCurrentBar) = ForceFloatRange (.15 * l_pd_I2(l_iCurrentBar) + .85 * l_pd_I2(l_iPreviousBar))
        l_pd_Q2(l_iCurrentBar) = ForceFloatRange (.15 * l_pd_Q2(l_iCurrentBar) + .85 * l_pd_Q2(l_iPreviousBar))

        ' Homodyne Discriminator
        ' Complex Conjugate Multiply
        l_pd_X1(l_iCurrentBar) = l_pd_I2(l_iCurrentBar) * l_pd_I2(l_iPreviousBar)
        l_pd_X2(l_iCurrentBar) = l_pd_I2(l_iCurrentBar) * l_pd_Q2(l_iPreviousBar)
        l_pd_Y1(l_iCurrentBar) = l_pd_Q2(l_iCurrentBar) * l_pd_Q2(l_iPreviousBar)
        l_pd_Y2(l_iCurrentBar) = l_pd_Q2(l_iCurrentBar) * l_pd_I2(l_iPreviousBar)
        l_pd_Re(l_iCurrentBar) = l_pd_X1(l_iCurrentBar) + l_pd_Y1(l_iCurrentBar)
        l_pd_Im(l_iCurrentBar) = l_pd_X2(l_iCurrentBar) - l_pd_Y2(l_iCurrentBar)

        ' Smooth to remove any undesired cross products
        l_pd_Re(l_iCurrentBar) = ForceFloatRange (.2 * l_pd_Re(l_iCurrentBar) + .8 * l_pd_Re(l_iPreviousBar))
        l_pd_Im(l_iCurrentBar) = ForceFloatRange (.2 * l_pd_Im(l_iCurrentBar) + .8 * l_pd_Im(l_iPreviousBar))

        ' Compute Cycle Period
        IF ( l_pd_Im(l_iCurrentBar) <> 0 AND l_pd_Re(l_iCurrentBar) <> 0 ) THEN
            @a_psPeriod[l_iCurrentBar] = (ForceFloatRange (360 / ( ATN(l_pd_Im(l_iCurrentBar) / l_pd_Re(l_iCurrentBar) ) _
                                            *(180.0 / PI))))
        END IF

        IF ( @a_psPeriod[l_iCurrentBar] > 1.5 * @a_psPeriod[l_iPreviousBar] ) THEN
            @a_psPeriod[l_iCurrentBar] = 1.5 * @a_psPeriod[l_iPreviousBar]
        END IF

        IF ( @a_psPeriod[l_iCurrentBar] < .67 * @a_psPeriod[l_iPreviousBar] ) THEN
            @a_psPeriod[l_iCurrentBar] = .67 * @a_psPeriod[l_iPreviousBar]
        END IF

        IF ( @a_psPeriod[l_iCurrentBar] < 6 ) THEN
            @a_psPeriod[l_iCurrentBar] = 6
        END IF

        IF ( @a_psPeriod[l_iCurrentBar] > 50 ) THEN
            @a_psPeriod[l_iCurrentBar] = 50
        END IF

        @a_psPeriod[l_iCurrentBar] = .2 * @a_psPeriod[l_iCurrentBar] + .8 * @a_psPeriod[l_iPreviousBar]

    NEXT l_iCurrentBar

    CalculateHilbertPeriod = l_bRtrn

END FUNCTION


'///////////////////////////////////////////////////////////////////////
'
' Function: ChannelHigh
'
' Description:
' The Enhanced Hilbert Channel Breakout System High Function
'
' Arguments:
' UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
' EntryK - Multiplier for Enhanced Hilbert Cycle Period
' EntryVal - Optional User-Defined Lookback Period
'
'///////////////////////////////////////////////////////////////////////


FUNCTION ChannelHigh SDECL ALIAS "ChannelHigh" (a_psDataRec AS MSXDataRec PTR, _
                                    a_psDataInfoArgs AS MSXDataInfoRecArgsArray PTR, _
                                    a_psNumericArgs AS MSXNumericArgsArray PTR, _
                                    a_psStringArgs AS MSXStringArgsArray PTR, _
                                    a_psCustomArgs AS MSXCustomArgsArray PTR, _
                                    a_psResultRec AS MSXResultRec PTR) EXPORT AS LONG
    LOCAL l_bRtrn AS LONG
    l_bRtrn = %MSX_SUCCESS

    ' Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
    LOCAL l_psPrice AS MSXDataInfoRec PTR
    LOCAL l_psHigh AS SINGLE PTR
    LOCAL l_iPeriodFirstValid AS INTEGER
    LOCAL l_iPeriodLastValid AS INTEGER
    LOCAL l_iPriceFirstValid AS INTEGER
    LOCAL l_iPriceLastValid AS INTEGER
    LOCAL l_iHighFirstValid AS INTEGER
    LOCAL l_iHighLastValid AS INTEGER
    LOCAL l_dEntryKInitial AS DOUBLE
    LOCAL l_dEntryValInitial AS DOUBLE
    LOCAL l_iArray_Size AS INTEGER
    LOCAL l_bResult AS DOUBLE
    LOCAL i AS INTEGER

    ' If the right number of args are passed in and the Price and High arrays are not empty
    IF ( @a_psDataInfoArgs.iNRecs = 1 AND _
         @a_psNumericArgs.iNRecs = 2 AND _
         @a_psStringArgs.iNRecs = 0 AND _
         @a_psCustomArgs.iNRecs = 0 AND _
         @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid >= 0 AND _
         @a_psDataInfoArgs.@psDataInfoRecs(0).iLastValid > @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid +6 AND _
         @a_psDataRec.sHigh.iFirstValid >= 0 AND _
         @a_psDataRec.sHigh.iLastValid > 0) THEN

        ' Assign a local pointer to the User-Defined Price data arg
        l_psPrice = @a_psDataInfoArgs.psDataInfoRecs(0)
        l_iPriceFirstValid = @l_psPrice.iFirstValid
        l_iPriceLastValid = @l_psPrice.iLastValid

        l_iPeriodFirstValid = @a_psDataRec.sHigh.iFirstValid
        l_iPeriodLastValid = @a_psDataRec.sHigh.iLastValid

        ' Allocate memory for Period Data
        l_iArray_Size = l_iPriceLastValid + 1
        DIM l_asPeriod(l_iArray_Size) AS SINGLE
        DIM l_psPeriod AS SINGLE PTR

        l_psPeriod = VARPTR(l_asPeriod(0))
        ' Calculate the Hilbert Cycle Period from the User-Defined Prices
        l_bResult = CalculateHilbertPeriod( @l_psPrice[0], _
                                            @l_psPeriod[0], _
                                            l_iArray_Size)

        ' Assign the High price data arg to the local array
        l_psHigh = @a_psDataRec.sHigh.pfValue
        l_iHighFirstValid = @a_psDataRec.sHigh.iFirstValid
        l_iHighLastValid = @a_psDataRec.sHigh.iLastValid

        ' Assign Numeric Args to local values
        l_dEntryKInitial = @a_psNumericArgs.fNumerics(0)
        l_dEntryValInitial = @a_psNumericArgs.fNumerics(1)

        ' If either manipulator is out of range, it is given a default value
        IF (l_dEntryKInitial > 100) THEN
            l_dEntryKInitial = 100
        END IF
        IF (l_dEntryValInitial > 100) THEN
            l_dEntryValInitial = 100
        END IF
        IF (l_dEntryKInitial < 0) THEN
            l_dEntryKInitial = 0
        END IF
        IF (l_dEntryValInitial < 0) THEN
            l_dEntryValInitial = 0
        END IF

        ' local calculation variables
        LOCAL l_iCurrentBar AS INTEGER
        LOCAL l_iCount AS INTEGER
        LOCAL l_dEntryVal AS DOUBLE
        LOCAL l_dEntryK AS DOUBLE
        LOCAL l_dEntryLookBack AS DOUBLE
        LOCAL l_psEntryChannel AS SINGLE PTR

        ' Allocate memory for l_asEntryChannel
        DIM l_asEntryChannel(l_iArray_Size) AS SINGLE

        ' Main Calculation loop
        FOR l_iCurrentBar = l_iPeriodFirstValid TO l_iPeriodLastValid

            ' Reinitialize variables for next iteration
            l_dEntryVal = l_dEntryValInitial
            l_dEntryK = l_dEntryKInitial
            l_dEntryLookBack = 0

            ' Calculate the Entry Look Back Period
            IF ( l_dEntryVal <> 0 ) THEN
                l_dEntryLookBack = l_dEntryVal
            ELSE
                l_dEntryLookBack = l_dEntryK * l_asPeriod(l_iCurrentBar)
            END IF

            IF ( l_dEntryLookBack < 1 ) THEN
                l_dEntryLookBack = 1
            END IF

            ' Set EntryChannel to the lowest possible value
            l_asEntryChannel(l_iCurrentBar) = 0

            ' Find the highest value in the Entry Look Back Period
            FOR l_iCount = l_iHighFirstValid TO l_dEntryLookBack
                IF ( l_iCurrentBar > l_iCount ) THEN
                    IF ( l_asEntryChannel(l_iCurrentBar) < @l_psHigh[l_iCurrentBar - l_iCount] ) THEN
                        l_asEntryChannel(l_iCurrentBar) = @l_psHigh[l_iCurrentBar - l_iCount]
                    END IF
                END IF
            NEXT l_iCount
        NEXT l_iCurrentBar

        ' Give back the Entry Channel
        ' Store results in the a_psResultRec structure
        FOR i = l_iPeriodFirstValid + 6 TO l_iPeriodLastValid
            @a_psResultRec.@psResultArray.@pfValue[i] = l_asEntryChannel(i)
        NEXT i
        @a_psResultRec.@psResultArray.iFirstValid = l_iPeriodFirstValid +6
        @a_psResultRec.@psResultArray.iLastValid = l_iPeriodLastValid

    ELSE
        ' We've recieved invalid arguments
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Invalid Argument(s)"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    IF (l_bResult <> %MSX_SUCCESS) THEN
        ' CalculateHilbertPeriod Function Failed
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Hilbert Period Calculation Failed"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    ChannelHigh = l_bRtrn
END FUNCTION


'///////////////////////////////////////////////////////////////////////
'
' Function: ChannelLow
'
' Description:
' The Enhanced Hilbert Channel Breakout System Low Function
'
' Arguments:
' UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
' ExitK - Multiplier for Enhanced Hilbert Cycle Period
' ExitVal - Optional User-Defined Lookback Period
'
'///////////////////////////////////////////////////////////////////////


FUNCTION ChannelLow SDECL ALIAS "ChannelLow" (a_psDataRec AS MSXDataRec PTR, _
                                    a_psDataInfoArgs AS MSXDataInfoRecArgsArray PTR, _
                                    a_psNumericArgs AS MSXNumericArgsArray PTR, _
                                    a_psStringArgs AS MSXStringArgsArray PTR, _
                                    a_psCustomArgs AS MSXCustomArgsArray PTR, _
                                    a_psResultRec AS MSXResultRec PTR) EXPORT AS LONG
    LOCAL l_bRtrn AS LONG
    l_bRtrn = %MSX_SUCCESS

    ' Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
    LOCAL l_psPrice AS MSXDataInfoRec PTR
    LOCAL l_psLow AS SINGLE PTR
    LOCAL l_iPeriodFirstValid AS INTEGER
    LOCAL l_iPeriodLastValid AS INTEGER
    LOCAL l_iPriceFirstValid AS INTEGER
    LOCAL l_iPriceLastValid AS INTEGER
    LOCAL l_iLowFirstValid AS INTEGER
    LOCAL l_iLowLastValid AS INTEGER
    LOCAL l_dExitKInitial AS DOUBLE
    LOCAL l_dExitValInitial AS DOUBLE
    LOCAL l_iArray_Size AS INTEGER
    LOCAL l_bResult AS DOUBLE
    LOCAL i AS INTEGER

    ' If the right number of args are passed in and the Price and High arrays are not empty
    IF ( @a_psDataInfoArgs.iNRecs = 1 AND _
         @a_psNumericArgs.iNRecs = 2 AND _
         @a_psStringArgs.iNRecs = 0 AND _
         @a_psCustomArgs.iNRecs = 0 AND _
         @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid >= 0 AND _
         @a_psDataInfoArgs.@psDataInfoRecs(0).iLastValid > @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid +6 AND _
         @a_psDataRec.sLow.iFirstValid >= 0 AND _
         @a_psDataRec.sLow.iLastValid > 0) THEN

        ' Assign a local pointer to the User-Defined Price data arg
        l_psPrice = @a_psDataInfoArgs.psDataInfoRecs(0)
        l_iPriceFirstValid = @a_psDataInfoArgs.@psDataInfoRecs(0).iFirstValid
        l_iPriceLastValid = @a_psDataInfoArgs.@psDataInfoRecs(0).iLastValid

        l_iPeriodFirstValid = @a_psDataRec.sHigh.iFirstValid
        l_iPeriodLastValid = @a_psDataRec.sHigh.iLastValid

        ' Allocate memory for Period Data
        l_iArray_Size = l_iPriceLastValid + 1
        DIM l_asPeriod(l_iArray_Size) AS SINGLE
        DIM l_psPeriod AS SINGLE PTR

        l_psPeriod = VARPTR(l_asPeriod(0))
        ' Calculate the Hilbert Cycle Period from the User-Defined Prices
        l_bResult = CalculateHilbertPeriod( @l_psPrice[0], _
                                            @l_psPeriod[0], _
                                            l_iArray_Size)

        ' Assign the High price data arg to the local array
        l_psLow = @a_psDataRec.sLow.pfValue
        l_iLowFirstValid = @a_psDataRec.sLow.iFirstValid
        l_iLowLastValid = @a_psDataRec.sLow.iLastValid

        ' Assign Numeric Args to local values
        l_dExitKInitial = @a_psNumericArgs.fNumerics(0)
        l_dExitValInitial = @a_psNumericArgs.fNumerics(1)

        ' If either manipulator is out of range, it is given a default value
        IF (l_dExitKInitial > 100) THEN
            l_dExitKInitial = 100
        END IF
        IF (l_dExitValInitial > 100) THEN
            l_dExitValInitial = 100
        END IF
        IF (l_dExitKInitial < 0) THEN
            l_dExitKInitial = 0
        END IF
        IF (l_dExitValInitial < 0) THEN
            l_dExitValInitial = 0
        END IF

        ' local calculation variables
        LOCAL l_iCurrentBar AS INTEGER
        LOCAL l_iCount AS INTEGER
        LOCAL l_dExitVal AS DOUBLE
        LOCAL l_dExitK AS DOUBLE
        LOCAL l_dExitLookBack AS DOUBLE

        ' Allocate memory for l_psEntryChannel
        DIM l_asExitChannel(l_iArray_Size) AS SINGLE

        ' Main Calculation loop
        FOR l_iCurrentBar = l_iPeriodFirstValid TO l_iPeriodLastValid

            ' Reinitialize variables for next iteration
            l_dExitVal = l_dExitValInitial
            l_dExitK = l_dExitKInitial
            l_dExitLookBack = 0

            ' Calculate the Exit Look Back Period
            IF ( l_dExitVal <> 0 ) THEN
                l_dExitLookBack = l_dExitVal
            ELSE
                l_dExitLookBack = l_dExitK * l_asPeriod(l_iCurrentBar)
            END IF

            IF ( l_dExitLookBack < 1 ) THEN
                l_dExitLookBack = 1
            END IF

            ' Set ExitChannel to a sufficiently high value
            l_asExitChannel(l_iCurrentBar) = 100000

            ' Find the lowest value in the Exit Look Back Period
            FOR l_iCount = l_iLowFirstValid TO l_dExitLookBack
                IF ( l_iCurrentBar > l_iCount ) THEN
                    IF ( l_asExitChannel(l_iCurrentBar) > @l_psLow[l_iCurrentBar - l_iCount] ) THEN
                        l_asExitChannel(l_iCurrentBar) = @l_psLow[l_iCurrentBar - l_iCount]
                    END IF
                END IF
            NEXT l_iCount
        NEXT l_iCurrentBar

        ' Give back the Exit Channel
        ' Store results in the a_psResultRec structure
        FOR i = l_iPeriodFirstValid + 6 TO l_iPeriodLastValid
            @a_psResultRec.@psResultArray.@pfValue[i] = l_asExitChannel(i)
        NEXT i
        @a_psResultRec.@psResultArray.iFirstValid = l_iPeriodFirstValid +6
        @a_psResultRec.@psResultArray.iLastValid = l_iPeriodLastValid

    ELSE
        ' We've recieved invalid arguments
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Invalid Argument(s)"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    IF (l_bResult <> %MSX_SUCCESS) THEN
        ' CalculateHilbertPeriod Function Failed
        l_bRtrn = %MSX_ERROR
        @a_psResultRec.szExtendedError = "Hilbert Period Calculation Failed"
        @a_psResultRec.@psResultArray.iFirstValid = 0
        @a_psResultRec.@psResultArray.iLastValid = -1
    END IF

    ChannelLow = l_bRtrn
END FUNCTION
                                                                                                                                                                                                                    