//============================================================================
// MetaStock MSX for the Enhanced Hilbert Cycle Period 
// and the Enhanced Hilbert Channel Breakout Signal High and Low Functions
//============================================================================

library EnhancedHilbert;   // Causes a .DLL file to be built

Uses SysUtils,    // Brings in szString funcs and exception trapping
     Math;        // Brings in various math functions

{$I MSXStruc.inc} // Required include - defines MSX Data Structures

//------------------------------------------------------------------------
// MSXInfo
//
// Required function.
//
//------------------------------------------------------------------------

function MSXInfo (var a_psDLLDef: MSXDLLDef): LongBool; stdcall;
begin

  StrLCopy (a_psDLLDef.szCopyright, 'Copyright (c) Equis International 2000',
            MSX_MAXSTRING-1);

  a_psDLLDef.iNFuncs := 3; // 3 functions will be exported.
  a_psDLLDef.iVersion := MSX_VERSION;
  MSXInfo := MSX_SUCCESS;
end;

//------------------------------------------------------------------------
// MSXNthFunction
//
// Required function.
// MSXNthFunction is called once for each function specified in the
//   MSXInfo function.  On each call, a_iNthFunc will indicate the
//   zero-based function to define.
// The value copied into a_sFuncDef.szFunctionName must EXACTLY match
//   the exported name of the function, including case.
//
//------------------------------------------------------------------------

function MSXNthFunction (a_iNthFunc: Integer;
                         var a_psFuncDef: MSXFuncDef): LongBool; stdcall;
  var l_bRtrn : LongBool;
begin
  l_bRtrn := MSX_SUCCESS;
  case a_iNthFunc of
	// One case entry (starting at 0) exists for each exported function.
    0: begin
         StrCopy(a_psFuncDef.szFunctionName, 'EnhancedHilbertPeriod');
         StrCopy(a_psFuncDef.szFunctionDescription, 'Enhanced Hilbert Cycle Period Function');
         a_psFuncDef.iNArguments := 1;
       end;
    1: begin
         StrCopy(a_psFuncDef.szFunctionName, 'ChannelHigh');
         StrCopy(a_psFuncDef.szFunctionDescription, 'Enhanced Hilbert Channel Breakout High Function');
         a_psFuncDef.iNArguments := 3;
       end;
    2: begin
         StrCopy(a_psFuncDef.szFunctionName, 'ChannelLow');
         StrCopy(a_psFuncDef.szFunctionDescription, 'Enhanced Hilbert Channel Breakout Low Function');
         a_psFuncDef.iNArguments := 3;
       end;
    else
      l_bRtrn := MSX_ERROR;
  end;
  MSXNthFunction := l_bRtrn;
end;

//------------------------------------------------------------------------
// MSXNthArg
//
// Required function only if any of your exported functions require
//   arguments. The functions and arguments are numbered
//   from 0.
//
//------------------------------------------------------------------------

function MSXNthArg(a_iNthFunc: Integer; a_iNthArg: Integer;
                   var a_psFuncArgDef: MSXFuncArgDef): LongBool; stdcall;
  var l_bRtrn : LongBool;
begin
  l_bRtrn := MSX_SUCCESS;
  a_psFuncArgDef.iNCustomStrings := 0;

  case a_iNthFunc of
    0:  // EnhancedHilbertPeriod
      case a_iNthArg of
        0: begin  // EnhancedHilbertPeriod:Arg0
                a_psFuncArgDef.iArgType := MSXDataArray;  // PriceDataArray
                StrCopy(a_psFuncArgDef.szArgName, 'PriceDataArray');
           end;
        else
           begin
                l_bRtrn := MSX_ERROR;
           end;
      end;
    1: // ChannelHigh
      case a_iNthArg of
        0: begin  // ChannelHigh:Arg0
               a_psFuncArgDef.iArgType := MSXDataArray;  // UserPriceArray
               StrCopy(a_psFuncArgDef.szArgName, 'UserPriceArray');
           end;
        1: begin  // ChannelHigh:Arg1
               a_psFuncArgDef.iArgType := MSXNumeric; // Multiplier
               StrCopy(a_psFuncArgDef.szArgName, 'PeriodMultiplier');
           end;
        2: begin  // ChannelHigh:Arg2
               a_psFuncArgDef.iArgType := MSXNumeric; // AlternatePeriod
               StrCopy(a_psFuncArgDef.szArgName, 'AlternatePeriod');
           end;
        else
           begin
               l_bRtrn := MSX_ERROR;
           end;
      end;
    2: // ChannelLow
      case a_iNthArg of
        0: begin  // ChannelLow:Arg0
                a_psFuncArgDef.iArgType := MSXDataArray;  // UserPriceArray
                StrCopy(a_psFuncArgDef.szArgName, 'UserPriceArray');
           end;
        1: begin  // ChannelLow:Arg1
                a_psFuncArgDef.iArgType := MSXNumeric; // Multiplier
                StrCopy(a_psFuncArgDef.szArgName, 'PeriodMultiplier');
           end;
        2: begin  // ChannelLow:Arg2
                a_psFuncArgDef.iArgType := MSXNumeric; // AlternatePeriod
                StrCopy(a_psFuncArgDef.szArgName, 'AlternatePeriod');
           end;
        else
           begin
                l_bRtrn := MSX_ERROR;
           end;
      end;
  else
    l_bRtrn := MSX_ERROR;
  end;

  MSXNthArg := l_bRtrn;
end;

// ************************************************************************
// This local utility function is used to help ensure that no overflows
//   or underflows will occur during calcuations.  The MSXTest program
//   Stress Test function will call your DLL with a wide range of values,
//   including positive and negative values of MaxSingle and MinSingle.
// Perform all intermediate calculations using Doubles and then force the
//   results into the range of a Single.
// ************************************************************************
function ForceSingleRange (a_lfDbl:Double) : Double;
begin
  if (a_lfDbl > 0.0) then
    begin
      if (a_lfDbl > MaxSingle) then
        a_lfDbl := MaxSingle  // make sure pos number <= MaxSingle
      else
        if (a_lfDbl < MinSingle) then
          a_lfDbl := MinSingle; // makesure pos number >= MinSingle
    end
  else
    if (a_lfDbl < 0.0) then
      begin
        if (a_lfDbl < -MaxSingle) then
          a_lfDbl := -MaxSingle  // make sure neg number >= MaxSingle
        else
          if (a_lfDbl > -MinSingle) then
            a_lfDbl := -MinSingle; // make sure neg number <= MinSingle
      end;
  ForceSingleRange := a_lfDbl;
end;

/////////////////////////////////////////////////////////////////////////
//
// Function: CalculateHilbertPeriod
//
// Description:
// Performs the actual calculation of the Hilbert Cycle Period
// called from the Period, High or Low functions
//
// Arguments:
// const Single a_psPrice - Pointer to Price data to use in calculation
// const Integer a_iFirstValid - First valid element in a_psPrice
// const Integer a_iLastValid - Last valid element in a_psPrice
// Single a_asPeriod - Array of Period data to be populated
// const Integer a_iArraySize - Actual size of a_asPeriod
//
/////////////////////////////////////////////////////////////////////////
function CalculateHilbertPeriod (const a_psPrice : PMSXSingleArray;
                                 const a_iFirstValid : Integer;
                                 const a_iLastValid : Integer;
                                 var a_asPeriod : array of Single;
                                 const a_iArray_Size : Integer): LongBool;
  var l_bRtrn : LongBool;
      // Declare local calculation variables
      l_iCurrentBar : Integer;
      l_iPreviousBar : Integer;  // Stores l_iCurrentBar - 1
      PI : Single;
      l_pdSmoother : array of Double;
      l_pdDetrender : array of Double;
      l_pd_I1 : array of Double;
      l_pd_Q1 : array of Double;
      l_pd_jI : array of Double;
      l_pd_jQ : array of Double;
      l_pd_I2 : array of Double;
      l_pd_Q2 : array of Double;
      l_pd_X1 : array of Double;
      l_pd_X2 : array of Double;
      l_pd_Y1 : array of Double;
      l_pd_Y2 : array of Double;
      l_pd_Re : array of Double;
      l_pd_Im : array of Double;

  begin
      PI := 3.14159265359;
      l_bRtrn := MSX_SUCCESS;

      // Allocate memory for all of the calculation arrays
      SetLength(l_pdSmoother, a_iArray_Size);
      SetLength(l_pdDetrender, a_iArray_Size);
      SetLength(l_pd_I1, a_iArray_Size);
      SetLength(l_pd_Q1, a_iArray_Size);
      SetLength(l_pd_jI, a_iArray_Size);
      SetLength(l_pd_jQ, a_iArray_Size);
      SetLength(l_pd_I2, a_iArray_Size);
      SetLength(l_pd_Q2, a_iArray_Size);
      SetLength(l_pd_X1, a_iArray_Size);
      SetLength(l_pd_X2, a_iArray_Size);
      SetLength(l_pd_Y1, a_iArray_Size);
      SetLength(l_pd_Y2, a_iArray_Size);
      SetLength(l_pd_Re, a_iArray_Size);
      SetLength(l_pd_Im, a_iArray_Size);

      // Main Calculation loop (begins after the fifth valid bar)
      for l_iCurrentBar := (a_iFirstValid + 5) to a_iLastValid do
        begin
	    // Set Previous Bar counter
	    l_iPreviousBar := l_iCurrentBar - 1;

	    // Calculate Smoother and Detrender values
	    l_pdSmoother[l_iCurrentBar] := (4 * a_psPrice[l_iCurrentBar] + 3 *
                                            a_psPrice[l_iPreviousBar] +
	    			            2 * a_psPrice[l_iCurrentBar - 2] +
                                            a_psPrice[l_iCurrentBar - 3])/10;
	    l_pdDetrender[l_iCurrentBar] := (0.25 * l_pdSmoother[l_iCurrentBar]
                                           + 0.75 * l_pdSmoother[l_iCurrentBar - 2]
                                           - 0.75 * l_pdSmoother[l_iCurrentBar - 4]
                                           - 0.25 * l_pdSmoother[l_iCurrentBar - 6])*
	    				 (0.046 * a_asPeriod[l_iPreviousBar] + 0.332);

	    // Calculate InPhase and Quadrature components
	    l_pd_Q1[l_iCurrentBar] := (0.25 * l_pdDetrender[l_iCurrentBar] + 0.75
                                     * l_pdDetrender[l_iCurrentBar - 2] -
	    			       0.75 * l_pdDetrender[l_iCurrentBar - 4]
                                     - 0.25 * l_pdDetrender[l_iCurrentBar - 6])
                                     * (0.046 * a_asPeriod[l_iPreviousBar] + 0.332);
	    l_pd_I1[l_iCurrentBar] := l_pdDetrender[l_iCurrentBar - 3];

	    // Advance the phase of l_pd_I1 and l_pd_Q1 by 90 degrees
	    l_pd_jI[l_iCurrentBar] := 0.25 * l_pd_I1[l_iCurrentBar] + 0.75
                                    * l_pd_I1[l_iCurrentBar - 2]
                                    - 0.75 * l_pd_I1[l_iCurrentBar - 4]
                                    - 0.25 * l_pd_I1[l_iCurrentBar - 6];
	    l_pd_jQ[l_iCurrentBar] := 0.25 * l_pd_Q1[l_iCurrentBar] + 0.75
                                    * l_pd_Q1[l_iCurrentBar - 2]
                                    - 0.75 * l_pd_Q1[l_iCurrentBar - 4] -0.25
                                    * l_pd_Q1[l_iCurrentBar - 6];

            // Phasor addition to equalize amplitude due to quadrature
	    // calculations (and 3 bar averaging)
	    l_pd_I2[l_iCurrentBar] := l_pd_I1[l_iCurrentBar] - l_pd_jQ[l_iCurrentBar];
	    l_pd_Q2[l_iCurrentBar] := l_pd_Q1[l_iCurrentBar] + l_pd_jI[l_iCurrentBar];

	    // Smooth the I and Q components before applying the discriminator
	    l_pd_I2[l_iCurrentBar] := ForceSingleRange (0.15 * l_pd_I2[l_iCurrentBar]
                                    + 0.85 * l_pd_I2[l_iPreviousBar]);
	    l_pd_Q2[l_iCurrentBar] := ForceSingleRange (0.15 * l_pd_Q2[l_iCurrentBar]
                                    + 0.85 * l_pd_Q2[l_iPreviousBar]);

	    // Homodyne Discriminator
	    // Complex Conjugate Multiply
	    l_pd_X1[l_iCurrentBar] := l_pd_I2[l_iCurrentBar] * l_pd_I2[l_iPreviousBar];
	    l_pd_X2[l_iCurrentBar] := l_pd_I2[l_iCurrentBar] * l_pd_Q2[l_iPreviousBar];
	    l_pd_Y1[l_iCurrentBar] := l_pd_Q2[l_iCurrentBar] * l_pd_Q2[l_iPreviousBar];
	    l_pd_Y2[l_iCurrentBar] := l_pd_Q2[l_iCurrentBar] * l_pd_I2[l_iPreviousBar];
	    l_pd_Re[l_iCurrentBar] := l_pd_X1[l_iCurrentBar] + l_pd_Y1[l_iCurrentBar];
	    l_pd_Im[l_iCurrentBar] := l_pd_X2[l_iCurrentBar] - l_pd_Y2[l_iCurrentBar];

	    // Smooth to remove any undesired cross products
	    l_pd_Re[l_iCurrentBar] := ForceSingleRange (0.2 * l_pd_Re[l_iCurrentBar]
                                    + 0.8 * l_pd_Re[l_iPreviousBar]);
	    l_pd_Im[l_iCurrentBar] := ForceSingleRange (0.2 * l_pd_Im[l_iCurrentBar]
                                    + 0.8 * l_pd_Im[l_iPreviousBar]);

	    // Compute Cycle Period
	    if (l_pd_Im[l_iCurrentBar] <> 0) and (l_pd_Re[l_iCurrentBar] <> 0 ) then
                a_asPeriod[l_iCurrentBar] := ForceSingleRange (360 /
                                           ( arctan(l_pd_Im[l_iCurrentBar] /
                                             l_pd_Re[l_iCurrentBar] ) *
                                           ( 180.0 / PI )));

	    if (a_asPeriod[l_iCurrentBar] > 1.5 * a_asPeriod[l_iPreviousBar] ) then
	        a_asPeriod[l_iCurrentBar] := 1.5 * a_asPeriod[l_iPreviousBar];

	    if (a_asPeriod[l_iCurrentBar] < 0.67 * a_asPeriod[l_iPreviousBar]) then
	        a_asPeriod[l_iCurrentBar] := 0.67 * a_asPeriod[l_iPreviousBar];

	    if (a_asPeriod[l_iCurrentBar] < 6) then
	        a_asPeriod[l_iCurrentBar] := 6;

	    if (a_asPeriod[l_iCurrentBar] > 50) then
	        a_asPeriod[l_iCurrentBar] := 50;

	    a_asPeriod[l_iCurrentBar] := 0.2 * a_asPeriod[l_iCurrentBar]
                                       + 0.8 * a_asPeriod[l_iPreviousBar];
        end;

      // Free up memory used by local variables
      l_pdSmoother := nil;
      l_pdDetrender := nil;
      l_pd_I1 := nil;
      l_pd_Q1 := nil;
      l_pd_jI := nil;
      l_pd_jQ := nil;
      l_pd_I2 := nil;
      l_pd_Q2 := nil;
      l_pd_X1 := nil;
      l_pd_X2 := nil;
      l_pd_Y1 := nil;
      l_pd_Y2 := nil;
      l_pd_Re := nil;
      l_pd_Im := nil;

      CalculateHilbertPeriod := l_bRtrn;
  end;


/////////////////////////////////////////////////////////////////////////
//
// Function: EnhancedHilbertPeriod
//
// Description:
// The Enhanced Hilbert Cycle Period Function used by the Enhanced Hilbert
// Channel Breakout system
//
// Arguments:
// PriceDataArray - Array of price data (high, low, close, etc.)
//                  to calculate cycle period from
//
/////////////////////////////////////////////////////////////////////////
function EnhancedHilbertPeriod (const a_psDataRec: PMSXDataRec;
                    const a_psDataInfoArgs: PMSXDataInfoRecArgsArray;
                    const a_psNumericArgs: PMSXNumericArgsArray;
                    const a_psStringArgs: PMSXStringArgsArray;
                    const a_psCustomArgs: PMSXCustomArgsArray;
                    var a_psResultRec: MSXResultRec): LongBool; stdcall;

var   l_bRtrn : LongBool;
      // Variables to hold a_psDataInfoArgs[0]
      l_asPeriod : array of Single;
      l_bResult : LongBool;
      l_iFirstValid : Integer;
      l_iLastValid : Integer;
      l_iArray_Size : Integer;
      i : Integer;

begin
  l_bRtrn := MSX_SUCCESS;
  l_bResult := MSX_SUCCESS;

  // if the right number of args are passed in and the array is not empty
  if (a_psDataInfoArgs.iNRecs = 1) and
     (a_psNumericArgs.iNRecs = 0) and
     (a_psStringArgs.iNRecs = 0) and
     (a_psCustomArgs.iNRecs = 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid >= 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iLastValid >
      a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid + 6) then
     begin
        // then assign the array to local values
        //l_psPrice := @a_psDataInfoArgs.psDataInfoRecs[0].pfValue;
	l_iFirstValid := a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid;
	l_iLastValid := a_psDataInfoArgs.psDataInfoRecs[0].iLastValid;
	l_iArray_Size := l_iLastValid + 1;
        SetLength(l_asPeriod, l_iArray_Size);

	//Call the Period Calculation Function
	l_bResult := CalculateHilbertPeriod(a_psDataInfoArgs.psDataInfoRecs[0].pfValue,
                                            l_iFirstValid,
                                            l_iLastValid,
                                            l_asPeriod,
                                            l_iArray_Size);

	// Store results in the a_psResultRec structure
        for i := (l_iFirstValid + 6) to (l_iLastValid + 1) do
           a_psResultRec.psResultArray.pfValue[i] := l_asPeriod[i];

	a_psResultRec.psResultArray.iFirstValid := l_iFirstValid + 6;
	a_psResultRec.psResultArray.iLastValid := l_iLastValid;

        // Free memory used by l_asPeriod array
	l_asPeriod := nil;

     end
  else
     begin
        // We've recieved invalid arguments
        // Invalidate the array
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Invalid Argument(s)',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  if l_bResult <> MSX_SUCCESS then
     begin
        // CalculateHilbertPeriod Function Failed
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Hilbert Period Calculation Failed',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  EnhancedHilbertPeriod := l_bRtrn;
end;


/////////////////////////////////////////////////////////////////////////
//
// Function: ChannelHigh
//
// Description:
// The Enhanced Hilbert Channel Breakout System High Function
//
// Arguments:
// UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
// EntryK - Multiplier for Enhanced Hilbert Cycle Period
// EntryVal - Optional User-Defined Lookback Period
//
/////////////////////////////////////////////////////////////////////////
function ChannelHigh (const a_psDataRec: PMSXDataRec;
                      const a_psDataInfoArgs: PMSXDataInfoRecArgsArray;
                      const a_psNumericArgs: PMSXNumericArgsArray;
                      const a_psStringArgs: PMSXStringArgsArray;
                      const a_psCustomArgs: PMSXCustomArgsArray;
                      var a_psResultRec: MSXResultRec): LongBool; stdcall;

var   l_bRtrn : LongBool;
      l_bResult : LongBool;

      // Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
      l_asPeriod : array of Single;
      l_asHigh : array of Single;
      l_iPeriodFirstValid : Integer;
      l_iPeriodLastValid : Integer;
      l_iPriceFirstValid : Integer;
      l_iPriceLastValid : Integer;
      l_iHighFirstValid : Integer;
      l_iHighLastValid : Integer;
      l_dEntryKInitial : Double;
      l_dEntryValInitial : Double;
      l_iArray_Size : Integer;

      // Local calculation variables
      l_iCurrentBar : Integer;
      l_iCount : Integer;
      l_dEntryVal : Double;
      l_dEntryK : Double;
      l_dEntryLookBack : Double;
      l_iEntryLookBack : Integer;
      l_asEntryChannel : array of Single;
      i : Integer;

begin

  l_bRtrn := MSX_SUCCESS;
  l_bResult := MSX_SUCCESS;

  // if the right number of args are passed in and the array is not empty
  if (a_psDataInfoArgs.iNRecs = 1) and
     (a_psNumericArgs.iNRecs = 2) and
     (a_psStringArgs.iNRecs = 0) and
     (a_psCustomArgs.iNRecs = 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid >= 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iLastValid >
      a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid + 6) and
     (a_psDataRec.sHigh.iFirstValid >= 0) and
     (a_psDataRec.sHigh.iLastValid > 0) then
     begin
        // then assign the array to local values
	l_iPriceFirstValid := a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid;
	l_iPriceLastValid := a_psDataInfoArgs.psDataInfoRecs[0].iLastValid;
	l_iArray_Size := l_iPriceLastValid + 1;
        SetLength(l_asPeriod, l_iArray_Size);
        l_iPeriodFirstValid := a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid;
        l_iPeriodLastValid := a_psDataInfoArgs.psDataInfoRecs[0].iLastValid;

	//Call the Period Calculation Function
	l_bResult := CalculateHilbertPeriod(a_psDataInfoArgs.psDataInfoRecs[0].pfValue,
                                            l_iPriceFirstValid,
                                            l_iPriceLastValid,
                                            l_asPeriod,
                                            l_iArray_Size);

	// Assign the High price data array upper and lower bounds
	l_iHighFirstValid := a_psDataRec.sHigh.iFirstValid;
	l_iHighLastValid := a_psDataRec.sHigh.iLastValid;

	// Assign Numeric Args to local values
	l_dEntryKInitial := a_psNumericArgs.fNumerics[0];
	l_dEntryValInitial := a_psNumericArgs.fNumerics[1];

	// If either manipulator is out of range, it is given a default value
	if l_dEntryKInitial > 100 then
		l_dEntryKInitial := 100;
	if l_dEntryValInitial > 100 then
		l_dEntryValInitial := 100;
	if l_dEntryKInitial < 0 then
		l_dEntryKInitial := 0;
	if l_dEntryValInitial < 0 then
		l_dEntryValInitial := 0;

        SetLength(l_asEntryChannel, (l_iPeriodLastValid + 1));
        SetLength(l_asHigh, l_iHighLastValid + 1);
        for l_iCount := l_iHighFirstValid to l_iHighLastValid do
                l_asHigh[l_iCount] := a_psDataRec.sHigh.pfValue[l_iCount];

	// Main Calculation Loop
	for l_iCurrentBar := l_iPeriodFirstValid to l_iPeriodLastValid do
           begin
	      // Reinitialize variables for next iteration
	      l_dEntryVal := l_dEntryValInitial;
	      l_dEntryK := l_dEntryKInitial;
	      l_dEntryLookBack := 0;

	      // Calculate the Entry Look Back Period
	      if l_dEntryVal <> 0 then
		 l_dEntryLookBack := l_dEntryVal
	      else
		 l_dEntryLookBack := ForceSingleRange(l_dEntryK * l_asPeriod[l_iCurrentBar]);

	      if l_dEntryLookBack < 1 then
		 l_dEntryLookBack := 1;

	      // Set EntryChannel to the lowest possible value
	      l_asEntryChannel[l_iCurrentBar] := 0;

              l_iEntryLookBack := Round(l_dEntryLookBack);

	      // Find the highest value in the Entry Look Back Period
	      for l_iCount := l_iHighFirstValid to l_iEntryLookBack + 1 do
	      	 begin
	      	    if l_iCurrentBar > l_iCount then
	      	       if l_asEntryChannel[l_iCurrentBar] < l_asHigh[l_iCurrentBar - l_iCount] then
	      		  l_asEntryChannel[l_iCurrentBar] := l_asHigh[l_iCurrentBar - l_iCount];
	      	 end;
	   end;
        // Give back the Entry Channel
        // Store results in the a_psResultRec structure
        for i := (l_iPeriodFirstValid + 6) to (l_iPeriodLastValid + 1) do
           a_psResultRec.psResultArray.pfValue[i] := l_asEntryChannel[i];

        a_psResultRec.psResultArray.iFirstValid := l_iPeriodFirstValid + 6;
        a_psResultRec.psResultArray.iLastValid := l_iPeriodLastValid;

        // Free up memory used by local variables
        l_asEntryChannel := nil;
        l_asPeriod := nil;
        l_asHigh := nil;
     end
  else
     begin
        // We've recieved invalid arguments
        // Invalidate the array
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Invalid Argument(s)',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  if l_bResult <> MSX_SUCCESS then
     begin
        // CalculateHilbertPeriod Function Failed
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Hilbert Period Calculation Failed',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  ChannelHigh := l_bRtrn;
end;


/////////////////////////////////////////////////////////////////////////
//
// Function: ChannelLow
//
// Description:
// The Enhanced Hilbert Channel Breakout System Low Function
//
// Arguments:
// UserDefinedPriceArray - Array of Price data chosen by the user "(HIGH+LOW)/2" etc.
// ExitK - Multiplier for Enhanced Hilbert Cycle Period
// ExitVal - Optional User-Defined Lookback Period
//
/////////////////////////////////////////////////////////////////////////
function ChannelLow (const a_psDataRec: PMSXDataRec;
                      const a_psDataInfoArgs: PMSXDataInfoRecArgsArray;
                      const a_psNumericArgs: PMSXNumericArgsArray;
                      const a_psStringArgs: PMSXStringArgsArray;
                      const a_psCustomArgs: PMSXCustomArgsArray;
                      var a_psResultRec: MSXResultRec): LongBool; stdcall;

var   l_bRtrn : LongBool;
      l_bResult : LongBool;

      // Variables to hold a_psDataInfoArgs[0] and a_psNumericArgs[0]
      l_asPeriod : array of Single;
      l_asLow : array of Single;
      l_iPeriodFirstValid : Integer;
      l_iPeriodLastValid : Integer;
      l_iPriceFirstValid : Integer;
      l_iPriceLastValid : Integer;
      l_iLowFirstValid : Integer;
      l_iLowLastValid : Integer;
      l_dExitKInitial : Double;
      l_dExitValInitial : Double;
      l_iArray_Size : Integer;

      // Local calculation variables
      l_iCurrentBar : Integer;
      l_iCount : Integer;
      l_dExitVal : Double;
      l_dExitK : Double;
      l_dExitLookBack : Double;
      l_iExitLookBack : Integer;
      l_asExitChannel : array of Single;
      i : Integer;

begin

  l_bRtrn := MSX_SUCCESS;
  l_bResult := MSX_SUCCESS;

  // if the right number of args are passed in and the array is not empty
  if (a_psDataInfoArgs.iNRecs = 1) and
     (a_psNumericArgs.iNRecs = 2) and
     (a_psStringArgs.iNRecs = 0) and
     (a_psCustomArgs.iNRecs = 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid >= 0) and
     (a_psDataInfoArgs.psDataInfoRecs[0].iLastValid >
      a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid + 6) and
     (a_psDataRec.sLow.iFirstValid >= 0) and
     (a_psDataRec.sLow.iLastValid > 0) then
     begin
        // then assign the array to local values
	l_iPriceFirstValid := a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid;
	l_iPriceLastValid := a_psDataInfoArgs.psDataInfoRecs[0].iLastValid;
	l_iArray_Size := l_iPriceLastValid + 1;
        SetLength(l_asPeriod, l_iArray_Size);
        l_iPeriodFirstValid := a_psDataInfoArgs.psDataInfoRecs[0].iFirstValid;
        l_iPeriodLastValid := a_psDataInfoArgs.psDataInfoRecs[0].iLastValid;

	//Call the Period Calculation Function
	l_bResult := CalculateHilbertPeriod(a_psDataInfoArgs.psDataInfoRecs[0].pfValue,
                                            l_iPriceFirstValid,
                                            l_iPriceLastValid,
                                            l_asPeriod,
                                            l_iArray_Size);

	// Assign the Low price data array upper and lower bounds
	l_iLowFirstValid := a_psDataRec.sLow.iFirstValid;
	l_iLowLastValid := a_psDataRec.sLow.iLastValid;

	// Assign Numeric Args to local values
	l_dExitKInitial := a_psNumericArgs.fNumerics[0];
	l_dExitValInitial := a_psNumericArgs.fNumerics[1];

	// If either manipulator is out of range, it is given a default value
	if l_dExitKInitial > 100 then
		l_dExitKInitial := 100;
	if l_dExitValInitial > 100 then
		l_dExitValInitial := 100;
	if l_dExitKInitial < 0 then
		l_dExitKInitial := 0;
	if l_dExitValInitial < 0 then
		l_dExitValInitial := 0;

        SetLength(l_asExitChannel, (l_iPeriodLastValid + 1));
        SetLength(l_asLow, (l_iLowLastValid + 1));
        for l_iCount := l_iLowFirstValid to l_iLowLastValid do
                l_asLow[l_iCount] := a_psDataRec.sLow.pfValue[l_iCount];

	// Main Calculation Loop
	for l_iCurrentBar := l_iPeriodFirstValid to l_iPeriodLastValid do
           begin
	      // Reinitialize variables for next iteration
	      l_dExitVal := l_dExitValInitial;
	      l_dExitK := l_dExitKInitial;
	      l_dExitLookBack := 0;

	      // Calculate the Exit Look Back Period
	      if l_dExitVal <> 0 then
		 l_dExitLookBack := l_dExitVal
	      else
		 l_dExitLookBack := ForceSingleRange(l_dExitK * l_asPeriod[l_iCurrentBar]);

	      if l_dExitLookBack < 1 then
		 l_dExitLookBack := 1;

	      // Set ExitChannel to a sufficiently high value
	      l_asExitChannel[l_iCurrentBar] := 100000;

              l_iExitLookBack := Round(l_dExitLookBack);

	      // Find the highest value in the Exit Look Back Period
	      for l_iCount := l_iLowFirstValid to l_iExitLookBack + 1 do
		 begin
		    if l_iCurrentBar > l_iCount then
		       if l_asExitChannel[l_iCurrentBar] > l_asLow[l_iCurrentBar - l_iCount] then
			  l_asExitChannel[l_iCurrentBar] := l_asLow[l_iCurrentBar - l_iCount];
		 end;
	   end;
        // Give back the Exit Channel
        // Store results in the a_psResultRec structure
        for i := (l_iPeriodFirstValid + 6) to (l_iPeriodLastValid + 1) do
           a_psResultRec.psResultArray.pfValue[i] := l_asExitChannel[i];

        a_psResultRec.psResultArray.iFirstValid := l_iPeriodFirstValid + 6;
        a_psResultRec.psResultArray.iLastValid := l_iPeriodLastValid;

        // Free up memory used by local variables
        l_asExitChannel := nil;
        l_asPeriod := nil;
        l_asLow := nil;
     end
  else
     begin
        // We've recieved invalid arguments
        // Invalidate the array
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Invalid Argument(s)',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  if l_bResult <> MSX_SUCCESS then
     begin
        // CalculateHilbertPeriod Function Failed
        a_psResultRec.psResultArray.iFirstValid := 0;
        a_psResultRec.psResultArray.iLastValid := -1;
        // Set the error message
        StrLCopy(a_psResultRec.szExtendedError, 'Hilbert Period Calculation Failed',
        MSX_MAXSTRING-1);
        l_bRtrn := MSX_ERROR;
     end;

  ChannelLow := l_bRtrn;
end;


exports
 MSXInfo,
 MSXNthFunction,
 MSXNthArg,
 EnhancedHilbertPeriod,
 ChannelHigh,
 ChannelLow;

begin

end.

